!= ®٤򸺿ꤵ뤿Υݥ
!
!= Sponge layer for damping velocity
!
! Authors::   Yasuhiro MORIKAWA
! Version::   $Id: phy_sponge_layer.f90,v 1.3 2008-06-01 16:06:53 morikawa Exp $
! Tag Name::  $Name: dcpam4-20080609-1 $
! Copyright:: Copyright (C) GFD Dennou Club, 2008. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]   
!

module phy_sponge_layer
  !
  != ®٤򸺿ꤵ뤿Υݥ
  !
  != Sponge layer for damping velocity
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !
  ! 굤 (ǥեȤǤ 1.0e4 Pa) ʲؤ, 
  ! ®٤ξʬ٤θθ̤򻻽Фޤ. 
  ! ζ, 
  !  (ǥեȤ 1 ) ʿͤ˹碌褦˷ޤ. 
  !
  ! Calculate effect of damping perturbation of velocity to layers under
  ! certain pressure (by default, 1.0e4 Pa). 
  ! Effect of damping is decided to 
  ! Strength of damping is decided to match to east and west mean 
  ! value by a certain time constant (default value is 1 day).
  !
  !== Procedures List
  !
  ! PhySpoLayCreate        :: PHYSPOLAY ѿν
  ! Damping                :: θ̤λ
  ! PhySpoLayClose         :: PHYSPOLAY ѿνλ
  ! PhySpoLayPutLine       :: PHYSPOLAY ѿ˳ǼƤΰ
  ! PhySpoLayinitialized   :: PHYSPOLAY ѿꤵƤ뤫ݤ
  ! PhySpoLaySetTime       :: 
  ! ------------  :: ------------
  ! PhySpoLayCreate        :: Constructor of "PHYSPOLAY"
  ! Damping                :: Calculation effect of damping
  ! PhySpoLayClose         :: Deconstructor of "PHYSPOLAY"
  ! PhySpoLayPutLine       :: Print information of "PHYSPOLAY"
  ! PhySpoLayinitialized   :: Check initialization of "PHYSPOLAY"
  ! PhySpoLaySetTime       :: Configure time
  !
  !== Usage
  !
  ! Ϥ, PHYSPOLAY ѿ, 
  ! PhySpoLayCreate ǽԤޤ.
  ! Damping ®, ®, ɽ̵Ϳ뤳Ȥ, 
  ! 줾ؤθθ̤򻻽Фޤ. 
  ! PHYSPOLAY ѿνλˤ
  ! PhySpoLayClose ѤƤ.
  !
  ! First, initialize "PHYSPOLAY" by "PhySpoLayCreate". 
  ! "Damping" receives zonal and meridional wind and surface pressure, 
  ! and returns effect of dumping them. 
  ! In order to terminate "PHYSPOLAY", use "PhySpoLayClose". 
  !

  use dc_types, only: DP, TOKEN, STRING
  use dc_date_types, only: DC_DIFFTIME
  use gt4_history_nmlinfo, only: GTHST_NMLINFO
  use intavr_operate, only: INTAVROPR
  implicit none
  private
  public:: PHYSPOLAY
  public:: PhySpoLayCreate, Damping
  public:: PhySpoLayClose, PhySpoLayPutLine
  public:: PhySpoLayInitialized, PhySpoLaySetTime

  type PHYSPOLAY
    !
    ! ޤ, PhySpoLayCreate  "PHYSPOLAY" 
    ! ѿꤷƲ. 
    ! ꤵ줿 "PHYSPOLAY" ѿѤݤˤ,
    ! PhySpoLayClose ˤäƽλԤäƤ.
    !
    ! Initialize "PHYSPOLAY" variable by 
    ! "PhySpoLayCreate" before usage.
    ! If you reuse "PHYSPOLAY" variable again for another application, 
    ! terminate by "PhySpoLayClose".
    !
    logical:: initialized = .false.
                              ! ե饰. 
                              ! Initialization flag

    !-----------------------------------------------------------------
    !  ʻȿ
    !  Grid points and maximum truncated wavenumber
    !-----------------------------------------------------------------
    integer:: imax            ! ٳʻ. 
                              ! Number of grid points in longitude
    integer:: jmax            ! ٳʻ. 
                              ! Number of grid points in latitude
    integer:: kmax            ! ľؿ. 
                              ! Number of vertical level

    !-----------------------------------------------------------------
    !  ǡ
    !  Axes data
    !-----------------------------------------------------------------
    real(DP), pointer:: x_Lon (:) =>null()
                              ! . Longitude
    real(DP), pointer:: x_Lon_Weight (:) =>null()
                              ! ʬѺɸŤ. 
                              ! Weight for integration in longitude
    real(DP), pointer:: y_Lat (:) =>null()
                              ! . Latitude
    real(DP), pointer:: y_Lat_Weight (:) =>null()
                              ! ʬѺɸŤ. 
                              ! Weight for integration in latitude
    real(DP), pointer:: z_Sigma (:) =>null()
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level
    real(DP), pointer:: z_DelSigma (:) =>null()
                              ! $ \Delta \sigma $ (). 
                              ! $ \Delta \sigma $ (Full)

    !-----------------------------------------------------------------
    !  
    !  Coefficients
    !-----------------------------------------------------------------
    real(DP):: DampingTime = 86400.0_DP
                              ! λ (ñ: ). 
                              ! Damping time constant (units: seconds)
    real(DP):: UpperPress = 0.0_DP
                              ! ϰϤξüε. 
                              ! Pressure in the top within the range of damping
    real(DP):: LowerPress = 1.0e4_DP
                              ! ϰϤβüε. 
                              ! Pressure at bottom within the range of damping

    !-----------------------------------------------------------------
    !  ʸǡ
    !  Character data
    !-----------------------------------------------------------------
!!$    character(TOKEN):: key00  ! . Keyword
!!$

    !-----------------------------------------------------------------
    !  
    !  Time control
    !-----------------------------------------------------------------
    type(DC_DIFFTIME):: current_time
                              ! ߻. Current time.
    type(DC_DIFFTIME):: delta_time
                              ! $ \Delta t $ . ॹƥå. Time step

    !-----------------------------------------------------------------
    !  ʿѱ黻ѥ⥸塼
    !  Module for average operation
    !-----------------------------------------------------------------
    type(INTAVROPR):: intavr_opr
                              ! intavr_operate ⥸塼Υ֥.
                              ! Object of "intavr_operate" module

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    !-----------------------------------------------------------------
    type(GTHST_NMLINFO):: gthstnml
                              ! NAMELIST#phy_sponge_layer_history_nml 
                              ! ꤵ̤ΥǡϾ. 
                              ! 
                              ! Individual data output information from 
                              ! "NAMELIST#phy_sponge_layer_history_nml". 
  end type PHYSPOLAY


  character(*), parameter:: version = &
    & '$Name: dcpam4-20080609-1 $' // &
    & '$Id: phy_sponge_layer.f90,v 1.3 2008-06-01 16:06:53 morikawa Exp $'

  !-----------------------------------------------------------------
  !  ³
  !  Public procedures
  !-----------------------------------------------------------------

  interface PhySpoLayCreate
    module procedure PhySpoLayCreate
  end interface

  interface Damping
    module procedure PhySpoLayDamping
  end interface

!!$  interface PhySpoLaySample
!!$    module procedure PhySpoLaySample
!!$  end interface

  interface PhySpoLayClose
    module procedure PhySpoLayClose
  end interface

  interface PhySpoLayPutLine
    module procedure PhySpoLayPutLine
  end interface

  interface PhySpoLayinitialized
    module procedure PhySpoLayInitialized
  end interface

  interface PhySpoLaySetTime
    module procedure PhySpoLaySetTime
  end interface

  !-----------------------------------------------------------------
  !  ³
  !  Private procedures
  !-----------------------------------------------------------------

  interface NmlRead
    module procedure PhySpoLayNmlRead
  end interface

contains

  subroutine PhySpoLayCreate( phy_spo_lay, &
    & imax, jmax, kmax, &
    & x_Lon, y_Lat, z_Sigma, &
    & DelTime, &
    & DampingTime, UpperPress, LowerPress, &
    & x_Lon_Weight, y_Lat_Weight, z_DelSigma, &
    & current_time_value, current_time_unit, &
    & history_varlist, &
    & history_interval_value, history_interval_unit, &
    & history_precision, history_fileprefix, &
    & nmlfile, err )
    !
    ! PHYSPOLAY ѿνԤޤ. 
    ! ¾Υ֥롼ѤɬΥ֥롼ˤä 
    ! PHYSPOLAY ѿꤷƤ. 
    !
    ! ʤ, Ϳ줿 *phy_spo_lay* ˽ꤵƤ, 
    ! ץϥ顼ȯޤ. 
    !
    ! NAMELIST Ѥˤϰ *nmlfile*  NAMELIST ե̾
    ! ͿƤ. NAMELIST ѿξܺ٤˴ؤƤ 
    ! NAMELIST#phy_sponge_layer_nml 򻲾ȤƤ. 
    !
    ! Constructor of "PHYSPOLAY". 
    ! Initialize *phy_spo_lay* by this subroutine, 
    ! before other procedures are used, 
    !
    ! Note that if *phy_spo_lay* is already initialized 
    ! by this procedure, error is occurred. 
    !
    ! In order to use NAMELIST, specify a NAMELIST filename to 
    ! argument *nmlfile*. See "NAMELIST#phy_sponge_layer_nml" 
    ! for details about a NAMELIST group. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_present, only: present_and_not_empty, present_and_true, &
      & present_select
    use dc_message, only: MessageNotify
    use dcpam_error, only: StoreError, DCPAM_ESMALLVAL
    use dc_error, only: DC_NOERR, DC_EALREADYINIT, &
      & DC_EARGLACK, DC_ENEGATIVE, DC_ENOFILEREAD, USR_ERRNO, HST_EBADVARNAME
    use dc_date, only: DCDiffTimeCreate
    use dc_hash, only: HASH, DCHashPut, DCHashRewind, DCHashNext, DCHashDelete
    use gt4_history_nmlinfo, only: HstNmlInfoCreate, HstNmlInfoAdd, &
      & HstNmlInfoEndDefine, HstNmlInfoPutLine, HstNmlInfoAllNameValid
    use gt4_history, only: GT_HISTORY, &
      & HistoryAddVariable, HistoryAddAttr
    use intavr_operate, only: IntAvrOprCreate
    implicit none
    type(PHYSPOLAY), intent(inout):: phy_spo_lay
    integer, intent(in):: imax ! ٳʻ. 
                               ! Number of grid points in longitude
    integer, intent(in):: jmax ! ٳʻ. 
                               ! Number of grid points in latitude
    integer, intent(in):: kmax ! ľؿ. 
                               ! Number of vertical level
    real(DP), intent(in):: x_Lon (0:imax-1)
                              ! . Longitude
    real(DP), intent(in):: y_Lat (0:jmax-1)
                              ! . Latitude
    real(DP), intent(in):: z_Sigma (0:kmax-1)
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level
    real(DP), intent(in):: DelTime    ! $ \Delta t $ . ॹƥå. Time step
    real(DP), intent(in), optional:: DampingTime
                              ! λ (ñ: ). 
                              ! Damping time constant (units: seconds)
    real(DP), intent(in), optional:: UpperPress
                              ! ϰϤξüε. 
                              ! Pressure in the top within the range of damping
    real(DP), intent(in), optional:: LowerPress
                              ! ϰϤβüε. 
                              ! Pressure at bottom within the range of damping
    real(DP), intent(in), optional:: x_Lon_Weight (0:imax-1)
                              ! ʬѺɸŤ. 
                              ! Weight for integration in longitude
    real(DP), intent(in), optional:: y_Lat_Weight (0:jmax-1)
                              ! ʬѺɸŤ. 
                              ! Weight for integration in latitude
    real(DP), intent(in), optional:: z_DelSigma (0:kmax-1)
                              ! $ \Delta \sigma $ (). 
                              ! $ \Delta \sigma $ (Full)
    real, intent(in), optional:: current_time_value
                              ! ߻ο. Numerical value of current time
    character(*), intent(in), optional:: current_time_unit
                              ! ߻ñ. Unit of current time
    character(*), intent(in), optional:: history_varlist
                              ! ҥȥǡνѿꥹ. 
                              ! ޤǶڤä¤٤. 
                              ! (: "Data1,Data2" ). 
                              ! 
                              ! List of variables output to history data. 
                              ! Delimiter is comma. 
                              ! (exp. "Data1,Data2" ). 
                              ! 
    real, intent(in), optional:: history_interval_value
                              ! ҥȥǡνϴֳ֤ο. 
                              ! Numerical value for interval of history data output
    character(*), intent(in), optional:: history_interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(*), intent(in), optional:: history_precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    character(*), intent(in), optional:: history_fileprefix
                              ! ҥȥǡΥե̾Ƭ. 
                              ! Prefix of history data filenames

    character(*), intent(in), optional:: nmlfile
                              ! NAMELIST ե̾. 
                              ! ΰ˶ʸʳͿ, 
                              ! ꤵ줿ե뤫 
                              ! NAMELIST ѿɤ߹ߤޤ. 
                              ! եɤ߹ʤˤϥ顼
                              ! ޤ. 
                              !
                              ! NAMELIST ѿξܺ٤˴ؤƤ 
                              ! NAMELIST#phy_sponge_layer_nml 
                              ! 򻲾ȤƤ. 
                              !
                              ! NAMELIST file name. 
                              ! If nonnull character is specified to 
                              ! this argument, 
                              ! NAMELIST group name is loaded from the 
                              ! file. 
                              ! If the file can not be read, 
                              ! an error occurs.
                              ! 
                              ! See "NAMELIST#phy_sponge_layer_nml" 
                              ! for details about a NAMELIST group. 
                              ! 

    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(STRING):: longname = ''
                              ! ѿεŪ̾. Descriptive name of variables
    character(STRING), allocatable:: dims(:)
                              ! ɸ̾. Name of axes
    character(STRING):: units = ''
                              ! ñ. Units
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module
    character(TOKEN):: precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    logical:: average
                              ! ϥǡʿѲե饰. 
                              ! Flag for average of output data
    type(HASH):: registered_varnames
                              ! Υ⥸塼뤫Ϥѿ̾Υꥹ.
                              ! 
                              ! List of names of variables output 
                              ! from this module. 
    logical:: hash_end
                              ! HASH ֥Ȥνλե饰. 
                              ! 
                              ! End flag of "HASH" object
    logical:: allvar_invalid
                              ! ̵ѿ̾Υåե饰. 
                              ! 
                              ! Check flag of invalid variable names
    character(STRING):: names_invalid
                              ! ̵ѿ̾. 
                              ! 
                              ! Invalid variable names
    character(STRING):: nmlfile_msg
                              ! NAMELIST ե̾˴ؤå. 
                              ! 
                              ! Messages about NAMELIST file name

    !-----------------------------------
    !  ѿ
    !  Work variables
    real(DP):: UpperPressCheck
                              ! ϰϤξüε. 
                              ! Pressure in the top within the range of cooling
    real(DP):: LowerPressCheck
                              ! ϰϤβüε. 
                              ! Pressure at bottom within the range of cooling

    real(DP), parameter:: PI = 3.1415926535897930_DP
                                ! $ \pi $ . ߼Ψ. Circular constant

    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'PhySpoLayCreate'
  continue
    call BeginSub( subname, version )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( phy_spo_lay % initialized ) then
      stat = DC_EALREADYINIT
      cause_c = 'PHYSPOLAY'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  Υå
    !  Validate arguments
    !-----------------------------------------------------------------
    if (imax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'imax'
      goto 999
    end if
    if (jmax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'jmax'
      goto 999
    end if
    if (kmax < 1) then
      stat = DC_ENEGATIVE
      cause_c = 'kmax'
      goto 999
    end if
    if (DelTime < 0.0_DP) then
      stat = DC_ENEGATIVE
      cause_c = 'DelTime'
      goto 999
    end if
    if ( present(DampingTime) ) then
      if (DampingTime < 0.0_DP) then
        stat = DC_ENEGATIVE
        cause_c = 'DampingTime'
        goto 999
      end if
    end if
    if ( present(UpperPress) ) then
      if (UpperPress < 0.0_DP) then
        stat = DC_ENEGATIVE
        cause_c = 'UpperPress'
        goto 999
      end if
      UpperPressCheck = UpperPress
    else
      UpperPressCheck = phy_spo_lay % UpperPress
    end if
    if ( present(LowerPress) ) then
      if (LowerPress < 0.0_DP) then
        stat = DC_ENEGATIVE
        cause_c = 'LowerPress'
        goto 999
      end if
      LowerPressCheck = LowerPress
    else
      LowerPressCheck = phy_spo_lay % LowerPress
    end if
    if ( UpperPressCheck > LowerPressCheck ) then
      call MessageNotify( 'W', subname, &
        & 'UpperPress=<%f> must be smaller than LowerPress=<%f>.', &
        & d = (/ UpperPressCheck, LowerPressCheck /) )
      stat = DCPAM_ESMALLVAL
      cause_c = 'UpperPress'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ȿʻ
    !  Configure wave number and grid point
    !-----------------------------------------------------------------
    phy_spo_lay % imax  = imax 
    phy_spo_lay % jmax  = jmax 
    phy_spo_lay % kmax  = kmax 

    !-----------------------------------------------------------------
    !  ɸ
    !  Configure axes
    !-----------------------------------------------------------------
    allocate( phy_spo_lay % x_Lon (0:imax-1) )
    phy_spo_lay % x_Lon = x_Lon

    allocate( phy_spo_lay % y_Lat (0:jmax-1) )
    phy_spo_lay % y_Lat = y_Lat

    allocate( phy_spo_lay % z_Sigma (0:kmax-1) )
    phy_spo_lay % z_Sigma = z_Sigma

    allocate( phy_spo_lay % x_Lon_Weight (0:imax-1) )
    if ( present(x_Lon_Weight) ) then
      phy_spo_lay % x_Lon_Weight = x_Lon_Weight
    else
      phy_spo_lay % x_Lon_Weight = 2.0_DP * PI / imax
    end if

    allocate( phy_spo_lay % y_Lat_Weight (0:jmax-1) )
    if ( present(y_Lat_Weight) ) then
      phy_spo_lay % y_Lat_Weight = y_Lat_Weight
    else
      phy_spo_lay % y_Lat_Weight = 2.0_DP / jmax
    end if

    allocate( phy_spo_lay % z_DelSigma (0:kmax-1) )
    if ( present(z_DelSigma) ) then
      phy_spo_lay % z_DelSigma = z_DelSigma
    else
      phy_spo_lay % z_DelSigma = 1.0_DP / kmax
    end if

    !-----------------------------------------------------------------
    !  ¾Υѥ᡼
    !  Other parameters are configured
    !-----------------------------------------------------------------
    if ( present(DampingTime) ) then
      phy_spo_lay % DampingTime = DampingTime
    end if

    if ( present(UpperPress) ) then
      phy_spo_lay % UpperPress = UpperPress
    end if

    if ( present(LowerPress) ) then
      phy_spo_lay % LowerPress = LowerPress
    end if

    !-----------------------------------------------------------------
    !  "intavr_operate" Υ֥Ȥν
    !  Initialization of object of "intavr_operate"
    !-----------------------------------------------------------------
    call IntAvrOprCreate( &
      & intavr_opr = phy_spo_lay % intavr_opr, &          ! (inout)
      & imax = imax, jmax = jmax, kmax = kmax, &          ! (in)
      & PI = PI, &                                        ! (in)
      & x_Lon_Weight = phy_spo_lay % x_Lon_Weight, &      ! (in)
      & y_Lat_Weight = phy_spo_lay % y_Lat_Weight, &      ! (in)
      & err = err )                                       ! (out)

    !-----------------------------------------------------------------
    !  
    !  Time control
    !-----------------------------------------------------------------
    if ( present(current_time_value) .and. present(current_time_unit) ) then
      call DCDiffTimeCreate( &
        & diff = phy_spo_lay % current_time, &  ! (out)
        & value = real( current_time_value, DP ), & ! (in)
        & unit = current_time_unit )                ! (in)
    else
      call DCDiffTimeCreate( &
        & diff = phy_spo_lay % current_time, & ! (out)
        & value = 0.0_DP, &    ! (in)
        & unit = 'sec' )       ! (in)
    end if

    call DCDiffTimeCreate( &
      & diff = phy_spo_lay % delta_time, & ! (out)
      & value = DelTime, &             ! (in)
      & unit = 'sec' )                 ! (in)

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    !-----------------------------------------------------------------

    !-------------------------
    !  ǥե
    !  Default values
    call HstNmlInfoCreate( gthstnml = phy_spo_lay % gthstnml ) ! (inout)

    !-------------------------
    !  ץʥ
    !  Values from optional arguments
    call HstNmlInfoAdd( &
      & gthstnml = phy_spo_lay % gthstnml, &   ! (inout)
      & name = '', &                               ! (in)
      & interval_value = history_interval_value, & ! (in)
      & interval_unit = history_interval_unit, &   ! (in)
      & precision = history_precision, &           ! (in)
      & average = .false., &                       ! (in)
      & fileprefix = history_fileprefix )          ! (in)

    if ( present(history_varlist) ) then
      call HstNmlInfoAdd( &
        & gthstnml = phy_spo_lay % gthstnml, & ! (inout)
        & name = history_varlist )                 ! (in)
    end if

    !-----------------------------------------------------------------
    !  NAMELIST ͤɤ߹
    !  Load values from NAMELIST
    !-----------------------------------------------------------------
    if ( present_and_not_empty(nmlfile) ) then
      call MessageNotify( 'M', subname, &
        & 'Loading NAMELIST file "%c" ...', &
        & c1 = trim(nmlfile) )
      call NmlRead ( nmlfile = nmlfile, &          ! (in)
        & DampingTime = phy_spo_lay % DampingTime, & ! (inout)
        & UpperPress  = phy_spo_lay % UpperPress, & ! (inout)
        & LowerPress  = phy_spo_lay % LowerPress, & ! (inout)
        & gthstnml    = phy_spo_lay % gthstnml, &  ! (inout)
        & err = err )                              ! (out)
      if ( present_and_true(err) ) then
        call MessageNotify( 'W', subname, &
          & '"%c" can not be read.', &
          & c1 = trim(nmlfile) )
        stat = DC_ENOFILEREAD
        cause_c = nmlfile
        goto 999
      end if
    end if

    call HstNmlInfoEndDefine( &
      & gthstnml = phy_spo_lay % gthstnml, & ! (inout)
      & err = err )                              ! (out)
    if ( present_and_true( err ) ) then
      stat = USR_ERRNO
      goto 999
    end if


    !-----------------------------------------------------------------
    !  ǡϤν
    !  Initialize data output
    !-----------------------------------------------------------------

    !-------------------------
    !  xyz_DUDtSpoDamping ν
    !  Configure the settings for "xyz_DUDtSpoDamping" output
    name = 'DUDtSpoDamping'
    longname = 'zonal wind tendency by damping in sponge layer'
    units = 'm s-2'
    allocate( dims(4) )
    dims = StoA( 'lon', 'lat', 'sig', 'time' )

    ! ϥեν.
    !   * gthist (gt4_history#GT_HISTORY) ꤵ.
    ! Initialize output file.
    !   * "gthist" (gt4_history#GT_HISTORY) is configured.
    call output_init  ! ֥롼. This is an internal subroutine
    deallocate( dims )

    !-------------------------
    !  xyz_DVDtSpoDamping ν
    !  Configure the settings for "xyz_DVDtSpoDamping" output
    name = 'DVDtSpoDamping'
    longname = 'meridional wind tendency by damping in sponge layer'
    units = 'm s-2'
    allocate( dims(4) )
    dims = StoA( 'lon', 'lat', 'sig', 'time' )

    ! ϥեν.
    !   * gthist (gt4_history#GT_HISTORY) ꤵ.
    ! Initialize output file.
    !   * "gthist" (gt4_history#GT_HISTORY) is configured.
    call output_init  ! ֥롼. This is an internal subroutine
    deallocate( dims )

!!$    !-------------------------
!!$    !  y_Data2 ν
!!$    !  Configure the settings for "y_Data2" output
!!$    name = 'Data2'
!!$    longname = 'Sample data (2)'
!!$    units = '1'
!!$    allocate( dims(2) )
!!$    dims = StoA( 'lat', 'time' )
!!$
!!$    ! ϥեν.
!!$    !   * gthist (gt4_history#GT_HISTORY) ꤵ.
!!$    ! Initialize output file.
!!$    !   * "gthist" (gt4_history#GT_HISTORY) is configured.
!!$    call output_init  ! ֥롼. This is an internal subroutine
!!$    ! °ղäʤɤԤˤϰʲΤ褦ˤ. 
!!$    ! Describe codes as follows in order to add attributes etc.
!!$    if ( associated( gthist ) ) then
!!$      call HistoryAddAttr( &
!!$        & history = gthist, &                           ! (inout)
!!$        & varname = name, attrname = 'missing_value', & ! (in)
!!$        &   value = 9999.0 )                            ! (in)
!!$    end if
!!$    deallocate( dims )

    !-----------------------------------------------------------------
    !  Υ⥸塼뤫Ϥѿ̾ΥꥹȤɽ
    !  Print list of names of variables output from this module
    !-----------------------------------------------------------------
    call Printf( STDOUT, &
      & ' *** MESSAGE *** +---- "%c" module output varnames list -----', &
      & c1 = 'phy_sponge_layer' )
    call DCHashRewind( hashv = registered_varnames ) ! (inout)
    do
      call DCHashNext( hashv = registered_varnames, &    ! (inout)
        & key = name, value = longname, end = hash_end ) ! (out)
      if ( hash_end ) exit
      call Printf( STDOUT, &
        & ' *** MESSAGE *** |      "%c" (%c)', &
        & c1 = trim(name), c2 = trim(longname) )
    enddo
    call DCHashDelete( hashv = registered_varnames ) ! (inout)
    call Printf( STDOUT, &
      & ' *** MESSAGE *** `----------------------------------------' )

    !-----------------------------------------------------------------
    !  ̵ѿ̾Υå
    !  Check invalid variable names
    !-----------------------------------------------------------------
    call HstNmlInfoAllNameValid( &
      & gthstnml = phy_spo_lay % gthstnml, &               ! (inout)
      & invalid = allvar_invalid, names = names_invalid, & ! (out)
      & err = err )                                        ! (out)

    if ( allvar_invalid ) then
      stat = HST_EBADVARNAME
      cause_c = names_invalid
      phy_spo_lay % initialized = .true.
      call PhySpoLayClose( phy_spo_lay, & ! (inout)
        & err = err )                     ! (out)
      if ( present(nmlfile) ) then
        nmlfile_msg = ' or NAMELIST "phy_sponge_layer_history_nml" in "'// trim(nmlfile) // '"'
      else
        nmlfile_msg = ''
      end if
      call MessageNotify( 'W', subname, &
        & 'names "%c" from an optional argument "history_varlist"%c are invalid.', &
        & c1 = trim(names_invalid), &
        & c2 = trim(nmlfile_msg) )
      goto 999
    end if

    !-----------------------------------------------------------------
    !  ͤΥå
    !  Validate setting values
    !-----------------------------------------------------------------
!!$    if ( phy_spo_lay % CoefAlpha < 0.0_DP ) then
!!$      stat = DC_ENEGATIVE
!!$      cause_c = 'CoefAlpha'
!!$      goto 999
!!$    end if

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
    phy_spo_lay % initialized = .true.
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  contains

    subroutine output_init
      !
      ! ѿ *name* ˴ؤƽϥեνԤޤ. 
      ! ϥե̾ϴֳ֤ʤɤξ phy_spo_lay % gthstnml 
      ! Фޤ. 
      ! 
      ! ѿ *name* ˴ؤƽϤԤˤ, 
      ! *gthist* ˽ե gt4_history#GT_HISTORY
      ! ѿ礵ޤ. Ǥʤ, *gthist* ֤ˤޤ. 
      !
      ! ޤ, ϥǡ٤ precision , 
      ! ϥǡʿѲβݤ average ꤷޤ. 
      ! 
      ! ɸϤɽѿꥹ *registered_varnames* 
      ! *name*, *longname*, *dims*, *units* Ͽޤ. 
      !
      ! An output file is initialized for a variable *name*. 
      ! Information such as the output filename and output intervals 
      ! is taken out of "phy_spo_lay % gthstnml". 
      !
      ! When output is done for the variable *name*, *gthist* is 
      ! associated with the "gt4_history#GT_HISTORY" variable of 
      ! the output file. Otherwise, *gthist* is nullified. 
      !
      ! Moreover, the accuracy of output data is set to *precision*, and 
      ! right or wrong of averaging the output data is set to *average*. 
      !
      ! *name*, *longname*, *dims*, *units* are registered to 
      ! a list of variables *registered_varnames* that is printed to 
      ! standard output. 
      !
      use dc_date, only: DCDiffTimeCreate, EvalSec, EvalByUnit
      use gt4_history_nmlinfo, only: GTHST_NMLINFO, &
        & HstNmlInfoInitialized, HstNmlInfoInquire, &
        & HstNmlInfoOutputValid, HstNmlInfoAssocGtHist, HstNmlInfoPutLine, &
        & HstNmlInfoSetValidName
      use gt4_history, only: GT_HISTORY, &
        & HistoryCreate, HistoryAddVariable, HistoryPut, &
        & HistoryAddAttr, HistoryInitialized

      !-----------------------------------
      !  ѿ
      !  Work variables
      character(STRING):: file
                                ! ҥȥǡΥե̾. 
                                ! History data filenames
      character(STRING):: dims_str
                                ! ɸΥꥹ. 
                                ! List of axes
      real:: interval_value
                                ! ҥȥǡνϴֳ֤ο. 
                                ! Numerical value for interval of history data output
      character(TOKEN):: interval_unit
                                ! ҥȥǡνϴֳ֤ñ. 
                                ! Unit for interval of history data output
    continue
      !-----------------------------------------------------------------
      !  ɸϤɽѿϿ
      !  Register a variable name for print to standard output
      !-----------------------------------------------------------------
      if ( allocated(dims) ) then
        dims_str = JoinChar( dims, ',' )
      else
        dims_str = ''
      end if
      call DCHashPut( hashv = registered_varnames, &     ! (inout)
        & key = name, &                                  ! (in)
        & value = trim( longname ) // ' [' // &
        &         trim( units ) // '] {' // &
        &         trim( dims_str ) // '}' ) ! (in)

      !-----------------------------------------------------------------
      !  ѿν
      !  Initialize variable
      !-----------------------------------------------------------------
      nullify( gthist )
      precision = 'float'
      average = .false.

      !-----------------------------------------------------------------
      !  ѿ̾ͭ
      !  Set validation of the variable name
      !-----------------------------------------------------------------
      call HstNmlInfoSetValidName( &
        & gthstnml = phy_spo_lay % gthstnml, & ! (in)
        & name = name )                        ! (in)

      !-----------------------------------------------------------------
      !  Ϥͭɤǧ
      !  Confirm whether the output is effective
      !-----------------------------------------------------------------
      if ( .not. HstNmlInfoOutputValid( phy_spo_lay % gthstnml, name ) ) then
        return
      end if

      !-----------------------------------------------------------------
      !  GT_HISTORY ѿμ
      !  Get "GT_HISTORY" variable
      !-----------------------------------------------------------------
      call HstNmlInfoAssocGtHist( &
        & gthstnml = phy_spo_lay % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & history = gthist, &                  ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      call HstNmlInfoInquire( &
        & gthstnml = phy_spo_lay % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & precision = precision, &             ! (out)
        & average = average, &                 ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      !-----------------------------------------------------------------
      !  GT_HISTORY ѿνγǧ
      !  Check initialization of "GT_HISTORY" variable
      !-----------------------------------------------------------------
      if ( HistoryInitialized( gthist ) ) then

        !---------------------------------------------------------------
        !  HistoryAddVariable ˤѿ
        !  A variable is created by "HistoryAddVariable"
        !---------------------------------------------------------------
        call HistoryAddVariable( &
          &  history = gthist, &                       ! (inout)
          &  varname = name,         dims = dims, &    ! (in)
          & longname = longname,    units = units, &   ! (in)
          &    xtype = precision, average = average )  ! (in)
        return
      end if

      !-----------------------------------------------------------------
      !  HistoryCreate Τͤμ
      !  Get the settings for "HistoryCreate"
      !-----------------------------------------------------------------
      call HstNmlInfoInquire( &
        & gthstnml = phy_spo_lay % gthstnml, & ! (in)
        & name = name, &                       ! (in)
        & file = file, &                       ! (out)
        & interval_unit = interval_unit, &     ! (out)
        & interval_value = interval_value, &   ! (out)
        & err = err )                          ! (out)
      if ( present_and_true( err ) ) return

      !-----------------------------------------------------------------
      !  HistoryCreate ˤե
      !  Files are created by "HistoryCreate"
      !-----------------------------------------------------------------
      call HistoryCreate( &
        & history = gthist, &   ! (out)
        & file = file, &        ! (in)
        & title = 'Sponge layer for damping velocity', &  ! (in)
        & source = 'dcpam4 project : ' // trim(version), &       ! (in)
        & institution = 'GFD Dennou Club', &                     ! (in)
        & dims = StoA( 'lon', 'lat', 'sig', 'time' ), &          ! (in)
        & dimsizes = (/ phy_spo_lay % imax, phy_spo_lay % jmax, &
        &               phy_spo_lay % kmax, 0 /), & ! (in)
        & longnames = StoA('longitude', 'latitude', &
        &                  'sigma at layer midpoints', &
        &                  'time'), &                            ! (in)
        & units = StoA( 'degree_east', 'degree_north', &
        &               '1', interval_unit ), &                  ! (in)
        & origin = real( EvalbyUnit( phy_spo_lay % current_time, &
        &                            interval_unit) ), &         ! (in)
        & interval = interval_value, &                           ! (in)
        & err = err )                                            ! (out)
      if ( present_and_true( err ) ) then
        nullify( gthist )
        return
      end if

      call HistoryAddAttr( &
        & history = gthist, &                            ! (inout)
        & varname = 'lon', attrname = 'standard_name', & ! (in)
        & value = 'longitude' )                          ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                            ! (inout)
        & varname = 'lat', attrname = 'standard_name', & ! (in)
        & value = 'latitude' )                           ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                              ! (inout)
        & varname = 'sig', attrname = 'standard_name', &   ! (in)
        & value = 'atmosphere_sigma_coordinate' )          ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                             ! (inout)
        & varname = 'time', attrname = 'standard_name', & ! (in)
        & value = 'time' )                                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                              ! (inout)
        & varname = 'sig', attrname = 'positive', &        ! (in)
        & value = 'down' )                                 ! (in)

      call HistoryPut( &
        & history = gthist, &                           ! (inout)
        & varname = 'lon', &                            ! (in)
        & array = phy_spo_lay % x_Lon / PI * 180.0_DP ) ! (in)
      call HistoryPut( &
        & history = gthist, &                           ! (inout)
        & varname = 'lat', &                            ! (in)
        & array = phy_spo_lay % y_Lat / PI * 180.0_DP ) ! (in)
      call HistoryPut( &
        & history = gthist, &             ! (inout)
        & varname = 'sig', &              ! (in)
        & array = phy_spo_lay % z_Sigma ) ! (in)

      call HistoryAddVariable( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lon_weight', &                           ! (in)
        & dims = StoA('lon'), &                               ! (in)
        & longname = 'weight for integration in longitude', & ! (in)
        & units = 'radian', xtype = 'double' )                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lon', attrname = 'gt_calc_weight', &     ! (in)
        & value = 'lon_weight' )                              ! (in)
      call HistoryPut( &
        & history = gthist, &                     ! (inout)
        & varname = 'lon_weight', &               ! (in)
        & array = phy_spo_lay % x_Lon_Weight )    ! (in)

      call HistoryAddVariable( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lat_weight', &                           ! (in)
        & dims = StoA('lat'), &                               ! (in)
        & longname = 'weight for integration in latitude', &  ! (in)
        & units = 'radian', xtype = 'double' )                ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                                 ! (inout)
        & varname = 'lat', attrname = 'gt_calc_weight', &     ! (in)
        & value = 'lat_weight' )                              ! (in)
      call HistoryPut( &
        & history = gthist, &                     ! (inout)
        & varname = 'lat_weight', &               ! (in)
        & array = phy_spo_lay % y_Lat_Weight )    ! (in)

      call HistoryAddVariable( &
        & history = gthist, &                                 ! (inout)
        & varname = 'sig_weight', &                           ! (in)
        & dims = StoA('sig'), &                               ! (in)
        & longname = 'weight for integration in sigma', &     ! (in)
        & units = '1', xtype = 'float' )                      ! (in)
      call HistoryAddAttr( &
        & history = gthist, &                                 ! (inout)
        & varname = 'sig', attrname = 'gt_calc_weight', &     ! (in)
        & value = 'sig_weight' )                              ! (in)
      call HistoryPut( &
        & history = gthist, &                     ! (inout)
        & varname = 'sig_weight', &               ! (in)
        & array = phy_spo_lay % z_DelSigma )      ! (in)

      !-----------------------------------------------------------------
      !  HistoryAddVariable ˤѿ
      !  A variable is created by "HistoryAddVariable"
      !-----------------------------------------------------------------
      if ( HistoryInitialized( gthist ) ) then
        call HistoryAddVariable( &
          &  history = gthist, &                       ! (inout)
          &  varname = name,         dims = dims, &    ! (in)
          & longname = longname,    units = units, &   ! (in)
          &    xtype = precision, average = average )  ! (in)
      else
        nullify( gthist )
      end if

    end subroutine output_init

  end subroutine PhySpoLayCreate

  subroutine PhySpoLayDamping( phy_spo_lay, &
    & xyz_U, xyz_V, xy_Ps, &
    & xyz_DUDtSpoDamping, xyz_DVDtSpoDamping, &
    & historyput_flag, &
    & err )
    !
    ! ® *xyz_U*, ® *xyz_V*  
    ! ɽ̵ *xy_Ps* , 
    ! θ̤ˤ®®ѲΨ
    ! *xyz_DUDtSpoDamping*, *xyz_DVDtSpoDamping* 
    ! 򻻽Ф֤ޤ. 
    ! ׻, *phy_spo_lay* λ򹹿ޤ. 
    !
    ! ʤ, Ϳ줿 *phy_spo_lay*  PhySpoLayCreate 
    ! ˤäƽꤵƤʤ, ץϥ顼ȯޤ. 
    !
    ! Calculate and return
    ! zonal wind and meridional tendency by damping in sponge layer
    ! *xyz_DUDtSpoDamping*, *xyz_DVDtSpoDamping* 
    ! from
    ! zonal wind *xyz_U* and meridional wind *xyz_V* and
    ! surface pressure *xy_Ps*.
    ! After the calculation, time stored in *phy_spo_lay* is updated. 
    !
    ! If *phy_spo_lay* is not initialized 
    ! by "PhySpoLayCreate" yet, error is occurred. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_present, only: present_and_true
    use dc_date, only: mod, operator(+), operator(==), EvalbyUnit, EvalSec
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT, USR_ERRNO
    use gt4_history_nmlinfo, only: HstNmlInfoInquire, &
      & HstNmlInfoOutputValid, HstNmlInfoAssocGtHist, HstNmlInfoPutLine
    use gt4_history, only: GT_HISTORY, HistoryPut, HistoryInitialized
    use intavr_operate, only: ya_AvrLon_xya
    implicit none
    type(PHYSPOLAY), intent(inout):: phy_spo_lay
    real(DP), intent(in):: xyz_U (0:phy_spo_lay%imax-1, 0:phy_spo_lay%jmax-1, 0:phy_spo_lay%kmax-1)
                              ! $ u $ .     ®. Zonal wind
    real(DP), intent(in):: xyz_V (0:phy_spo_lay%imax-1, 0:phy_spo_lay%jmax-1, 0:phy_spo_lay%kmax-1)
                              ! $ v $ .     ®. Meridional wind
    real(DP), intent(in):: xy_Ps  (0:phy_spo_lay%imax-1, 0:phy_spo_lay%jmax-1)
                              ! $ p_s $ .   ɽ̵. Surface pressure

    real(DP), intent(out):: xyz_DUDtSpoDamping (0:phy_spo_lay%imax-1, 0:phy_spo_lay%jmax-1, 0:phy_spo_lay%kmax-1)
                              ! $ \DP{u}{t} $ . 
                              ! θ̤ˤ®Ѳ. 
                              ! Zonal wind tendency by damping effect
    real(DP), intent(out):: xyz_DVDtSpoDamping (0:phy_spo_lay%imax-1, 0:phy_spo_lay%jmax-1, 0:phy_spo_lay%kmax-1)
                              ! $ \DP{v}{t} $ . 
                              ! θ̤ˤ®Ѳ. 
                              ! Meridional wind tendency by damping effect
    logical, intent(in), optional:: historyput_flag
                              ! ǡϤΥե饰. 
                              ! SetTime ˤäƻŪ
                              ! ꤷˤ, ΰ
                              ! .true. ޤ .false. ꤹ
                              ! ȤǥǡϤΥ󥪥դ
                              ! Ū˻ꤹɬפޤ. 
                              ! ǥեȤ .false. Ǥ. 
                              ! 
                              ! Data output flag. 
                              ! When time is specified by "SetTime", 
                              ! explicit specification of data output 
                              ! on/off by specifying ".true." or ".false." 
                              ! to this argument. 
                              ! Default value is ".false.". 
                              ! 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  *phy_spo_lay* Ф
    !  Setting values fetched from *phy_spo_lay*
    integer:: imax            ! ٳʻ. 
                              ! Number of grid points in longitude
    integer:: jmax            ! ٳʻ. 
                              ! Number of grid points in latitude
    integer:: kmax            ! ľؿ. 
                              ! Number of vertical level

    real(DP):: z_Sigma (0:phy_spo_lay%kmax-1)
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level

    real(DP):: DampingTime
                              ! λ (ñ: ). 
                              ! Damping time constant (units: seconds)
    real(DP):: UpperPress
                              ! ϰϤξüε. 
                              ! Pressure in the top within the range of damping
    real(DP):: LowerPress
                              ! ϰϤβüε. 
                              ! Pressure at bottom within the range of damping

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    real:: time
                              ! . Time
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    real(DP):: xyz_Press (0:phy_spo_lay%imax-1, 0:phy_spo_lay%jmax-1, 0:phy_spo_lay%kmax-1)
                              ! $ p $ .     . Air pressure
    real(DP):: yz_UAvrLon (0:phy_spo_lay%jmax-1, 0:phy_spo_lay%kmax-1)
                              ! ®ʿ
                              ! Zonal mean zonal wind
    real(DP):: yz_VAvrLon (0:phy_spo_lay%jmax-1, 0:phy_spo_lay%kmax-1)
                              ! ®ʿ
                              ! Zonal mean meritional wind

    integer:: i, j, k         ! DO 롼Ѻѿ
                              ! Work variables for DO loop

    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'PhySpoLayDamping'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( .not. phy_spo_lay % initialized ) then
      stat = DC_ENOTINIT
      cause_c = 'PHYSPOLAY'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  *phy_spo_lay* ˳ǼƤͤμФ
    !  Fetch setting values stored in *phy_spo_lay*
    !-----------------------------------------------------------------
    imax  = phy_spo_lay % imax 
    jmax  = phy_spo_lay % jmax 
    kmax  = phy_spo_lay % kmax 

    z_Sigma  =  phy_spo_lay % z_Sigma 

    DampingTime = phy_spo_lay % DampingTime 
    UpperPress  = phy_spo_lay % UpperPress 
    LowerPress  = phy_spo_lay % LowerPress 

    !-----------------------------------------------------------------
    !  Ѳα黻
    !  Calculate tendency
    !-----------------------------------------------------------------
    do k = 0, kmax-1
      xyz_Press(:,:,k) = xy_Ps * z_Sigma(k)
    end do

    yz_UAvrLon = ya_AvrLon_xya( xyz_U, phy_spo_lay % intavr_opr )
    yz_VAvrLon = ya_AvrLon_xya( xyz_V, phy_spo_lay % intavr_opr )

    xyz_DUDtSpoDamping = 0.0_DP
    xyz_DVDtSpoDamping = 0.0_DP

    do k = 0, kmax-1
      do j = 0, jmax-1
        do i = 0, imax-1
          if ( ( xyz_Press(i,j,k) < LowerPress ) &
            &  .and. ( xyz_Press(i,j,k) > UpperPress ) ) then
            xyz_DUDtSpoDamping(i,j,k) = &
              & ( yz_UAvrLon(j,k) - xyz_U(i,j,k) ) / DampingTime
            xyz_DVDtSpoDamping(i,j,k) = &
              & ( yz_VAvrLon(j,k) - xyz_V(i,j,k) ) / DampingTime
          end if
        end do
      end do
    end do

    !----------------------------------------------------------------
    !  ҥȥեؤΥǡ
    !  History data output
    !----------------------------------------------------------------

    !-----------------------------------
    !  xyz_DUDtSpoDamping ν
    !  Output "xyz_DUDtSpoDamping"
    name = 'DUDtSpoDamping'

    ! ϤΥå. 
    !   * gthist (gt4_history#GT_HISTORY), time (ñټ¿) ꤵ.
    ! Check for output.
    !   * "gthist" (gt4_history#GT_HISTORY), time (real) are configured.
    call output_check ! ֥롼. This is an internal subroutine

    ! ϥǡ array Ϥ.
    ! Give output data to argument "array"
    if ( associated( gthist ) ) then
      call HistoryPut( &
        & history = gthist, &                ! (inout)
        & varname = name, array = xyz_DUDtSpoDamping, & ! (in)
        & time = time, quiet = .false., &    ! (in)
        & err = err )                        ! (out)
    end if

    !-----------------------------------
    !  xyz_DVDtSpoDamping ν
    !  Output "xyz_DVDtSpoDamping"
    name = 'DVDtSpoDamping'

    ! ϤΥå. 
    !   * gthist (gt4_history#GT_HISTORY), time (ñټ¿) ꤵ.
    ! Check for output.
    !   * "gthist" (gt4_history#GT_HISTORY), time (real) are configured.
    call output_check ! ֥롼. This is an internal subroutine

    ! ϥǡ array Ϥ.
    ! Give output data to argument "array"
    if ( associated( gthist ) ) then
      call HistoryPut( &
        & history = gthist, &                ! (inout)
        & varname = name, array = xyz_DVDtSpoDamping, & ! (in)
        & time = time, quiet = .false., &    ! (in)
        & err = err )                        ! (out)
    end if

!!$    !-----------------------------------
!!$    !  y_Data2 ν
!!$    !  Output "y_Data2"
!!$    name = 'Data2'
!!$
!!$    ! ϤΥå. 
!!$    !   * gthist (gt4_history#GT_HISTORY), time (ñټ¿) ꤵ.
!!$    ! Check for output.
!!$    !   * "gthist" (gt4_history#GT_HISTORY), time (real) are configured.
!!$    call output_check ! ֥롼. This is an internal subroutine
!!$
!!$    ! ϥǡ array Ϥ.
!!$    ! Give output data to argument "array"
!!$    if ( associated( gthist ) ) then
!!$      call HistoryPut( &
!!$        & history = gthist, &                ! (inout)
!!$        & varname = name, array = y_Data2, & ! (in)
!!$        & time = time, quiet = .false., &    ! (in)
!!$        & err = err )                        ! (out)
!!$    end if

    !-----------------------------------------------------------------
    !  ι
    !  Update time
    !-----------------------------------------------------------------
    phy_spo_lay % current_time = &
      & phy_spo_lay % current_time + phy_spo_lay % delta_time

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  contains

    subroutine output_check
      !
      ! ѿ *name* Ϥ뤫ɤåޤ. 
      ! Ϥ˴ؤ dcm_sam_cod % gthstnml Фޤ. 
      !
      ! ѿ *name* ˴ؤƽϤ褦ꤵƤˤ, 
      ! *gthist* ˽ե gt4_history#GT_HISTORY
      ! ѿ礵ޤ. Ǥʤ, *gthist* ֤ˤޤ. 
      !
      ! ޤ, ߻ *time* ꤷޤ. 
      !
      ! Check whether to output variable *name*. 
      ! Information about output is taken out of "dcm_sam_cod % gthstnml". 
      !
      ! When output is done for the variable *name*, *gthist* is 
      ! associated with "gt4_history#GT_HISTORY" variable of
      ! the output file. Otherwise, *gthist* is nullified. 
      !
      ! Moreover, current time is set to *time*. 
      !
      character(TOKEN):: interval_unit
                                ! ҥȥǡνϴֳ֤ñ. 
                                ! Unit for interval of history data output
    continue

      nullify( gthist )
      time = 0.0

      if ( HstNmlInfoOutputValid( phy_spo_lay % gthstnml, name ) ) then
        call HstNmlInfoInquire( &
          & gthstnml = phy_spo_lay % gthstnml, & ! (in)
          & name = name, &                           ! (in)
          & interval_unit = interval_unit )          ! (out)

        time = real( EvalbyUnit( phy_spo_lay % current_time, interval_unit ) )
        if ( present_and_true( historyput_flag ) ) time = 0.0

        call HstNmlInfoAssocGtHist( & 
          & gthstnml = phy_spo_lay % gthstnml, & ! (in)
          & name = name, &                           ! (in)
          & history = gthist, err = err )            ! (out)
        if ( present_and_true( err ) ) then
          nullify( gthist )
          return
        end if

        if ( .not. HistoryInitialized( gthist ) ) nullify( gthist )
      end if

    end subroutine output_check
  end subroutine PhySpoLayDamping

!!$  subroutine PhySpoLaySample( phy_spo_lay, err )
!!$    !--
!!$    ! PhySpoLaySample 򵭽ҤƤ. 
!!$    !++
!!$    ! ʤ, Ϳ줿 *phy_spo_lay*  PhySpoLayCreate 
!!$    ! ˤäƽꤵƤʤ, ץϥ顼ȯޤ. 
!!$    !--
!!$    ! Describe brief of "PhySpoLaySample". 
!!$    !++
!!$    ! If *phy_spo_lay* is not initialized 
!!$    ! by "PhySpoLayCreate" yet, error is occurred.
!!$    !
!!$    use dc_trace, only: BeginSub, EndSub
!!$    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
!!$    use dc_types, only: DP, STRING, TOKEN, STDOUT
!!$    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
!!$    implicit none
!!$    type(PHYSPOLAY), intent(inout):: phy_spo_lay
!!$    logical, intent(out), optional:: err
!!$                              ! 㳰ѥե饰. 
!!$                              ! ǥեȤǤ, μ³ǥ顼
!!$                              ! , ץ϶λޤ. 
!!$                              !  *err* Ϳ, 
!!$                              ! ץ϶λ, 
!!$                              ! *err*  .true. ޤ. 
!!$                              !
!!$                              ! Exception handling flag. 
!!$                              ! By default, when error occur in 
!!$                              ! this procedure, the program aborts. 
!!$                              ! If this *err* argument is given, 
!!$                              ! .true. is substituted to *err* and 
!!$                              ! the program does not abort. 
!!$
!!$    !-----------------------------------
!!$    !  *phy_spo_lay* Ф
!!$    !  Setting values fetched from *phy_spo_lay*
!!$!!$    real(DP):: CoefAlpha      ! $ \alpha $ . . Coefficient
!!$!!$    real(DP):: DelTime        ! $ \Delta t $ . ॹƥå. Time step
!!$
!!$    !-----------------------------------
!!$    !  ѿ
!!$    !  Work variables
!!$    integer:: stat
!!$    character(STRING):: cause_c
!!$    character(*), parameter:: subname = 'PhySpoLaySample'
!!$  continue
!!$    call BeginSub( subname )
!!$    stat = DC_NOERR
!!$    cause_c = ''
!!$
!!$    !-----------------------------------------------------------------
!!$    !  Υå
!!$    !  Check initialization
!!$    !-----------------------------------------------------------------
!!$    if ( .not. phy_spo_lay % initialized ) then
!!$      stat = DC_ENOTINIT
!!$      cause_c = 'PHYSPOLAY'
!!$      goto 999
!!$    end if
!!$
!!$    !-----------------------------------------------------------------
!!$    !  *phy_spo_lay* ˳ǼƤͤμФ
!!$    !  Fetch setting values stored in *phy_spo_lay*
!!$    !-----------------------------------------------------------------
!!$!!$    CoefAlpha = phy_spo_lay % CoefAlpha
!!$!!$    DelTime   = phy_spo_lay % DelTime
!!$
!!$    !-----------------------------------------------------------------
!!$    !  λ, 㳰
!!$    !  Termination and Exception handling
!!$    !-----------------------------------------------------------------
!!$999 continue
!!$    call StoreError( stat, subname, err, cause_c )
!!$    call EndSub( subname )
!!$
!!$  end subroutine PhySpoLaySample

  subroutine PhySpoLayClose( phy_spo_lay, err )
    !
    ! PHYSPOLAY ѿνλԤޤ. 
    ! ʤ, Ϳ줿 *phy_spo_lay*  PhySpoLayCreate 
    ! ˤäƽꤵƤʤ, ץϥ顼ȯޤ. 
    !
    ! Deconstructor of "PHYSPOLAY". 
    ! Note that if *phy_spo_lay* is not initialized 
    ! by "PhySpoLayCreate" yet, error is occurred. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
    use gt4_history_nmlinfo, only: HstNmlInfoClose, HstNmlInfoNames, &
      & HstNmlInfoAssocGtHist, HstNmlInfoPutLine
    use gt4_history, only: GT_HISTORY, HistoryClose, HistoryInitialized
    use intavr_operate, only: IntAvrOprClose
    implicit none
    type(PHYSPOLAY), intent(inout):: phy_spo_lay
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(STRING):: varnames
                              ! ѿ̾ꥹ. 
                              ! List of variables
    character(TOKEN), pointer:: varnames_array(:) =>null()
                              ! ѿ̾ꥹ. 
                              ! List of variables (array) 
    integer:: i, vnmax
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'PhySpoLayClose'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( .not. phy_spo_lay % initialized ) then
      stat = DC_ENOTINIT
      cause_c = 'PHYSPOLAY'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  "PHYSPOLAY" ξõ
    !  Clear the settings for "PHYSPOLAY"
    !-----------------------------------------------------------------
    deallocate( phy_spo_lay % x_Lon )
    deallocate( phy_spo_lay % x_Lon_Weight )
    deallocate( phy_spo_lay % y_Lat )
    deallocate( phy_spo_lay % y_Lat_Weight )
    deallocate( phy_spo_lay % z_Sigma )
    deallocate( phy_spo_lay % z_DelSigma )
    phy_spo_lay % DampingTime = 86400.0_DP
    phy_spo_lay % UpperPress = 0.0_DP
    phy_spo_lay % LowerPress = 1.0e4_DP

    call IntAvrOprClose( intavr_opr = phy_spo_lay % intavr_opr )

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡϤνλ
    !  Terminate the settings for history data output
    !-----------------------------------------------------------------
    varnames = HstNmlInfoNames( phy_spo_lay % gthstnml )
    call Split( str = varnames, sep = ',', & ! (in)
      & carray = varnames_array )            ! (out)
    vnmax = size( varnames_array )

    do i = 1, vnmax
      name = varnames_array(i)
      if ( trim( name ) == '' ) exit
      nullify( gthist )
      call HstNmlInfoAssocGtHist( & 
        & gthstnml = phy_spo_lay % gthstnml, & ! (in)
        & name = name, &                           ! (in)
        & history = gthist, &                      ! (out)
        & err = err )                              ! (out)
      if ( HistoryInitialized( gthist ) ) then

        call HistoryClose( history = gthist, & ! (inout)
          & err = err )                        ! (out)

      end if
    end do

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡγղ
    !  Deallocate the settings for history data output
    !-----------------------------------------------------------------
    call HstNmlInfoClose( &
      & phy_spo_lay % gthstnml, & ! (inout)
      & err = err )                   ! (out)

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
    phy_spo_lay % initialized = .false.
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  end subroutine PhySpoLayClose

  subroutine PhySpoLayPutLine( phy_spo_lay, unit, indent, err )
    !
    !  *phy_spo_lay* ꤵƤޤ. 
    ! ǥեȤǤϥåɸϤ˽Ϥޤ. 
    ! *unit* ֹꤹ뤳Ȥ, ѹ뤳ȤǽǤ. 
    !
    ! Print information of *phy_spo_lay*. 
    ! By default messages are output to standard output. 
    ! Unit number for output can be changed by *unit* argument. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_date, only: EvalSec
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
    use gt4_history_nmlinfo, only: GTHST_NMLINFO, HstNmlInfoPutLine
    use intavr_operate, only: IntAvrOprPutLine
    implicit none
    type(PHYSPOLAY), intent(in):: phy_spo_lay
    integer, intent(in), optional:: unit
                              ! ֹ. 
                              ! ǥեȤνɸ. 
                              !
                              ! Unit number for output. 
                              ! Default value is standard output. 
    character(*), intent(in), optional:: indent
                              ! ɽåλ. 
                              !
                              ! Indent of displayed messages. 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    integer:: out_unit
    integer:: indent_len
    character(STRING):: indent_str
    character(*), parameter:: subname = 'PhySpoLayPutLine'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  ֹȻ
    !  Configure output unit number and indents
    !-----------------------------------------------------------------
    if ( present(unit) ) then
      out_unit = unit
    else
      out_unit = STDOUT
    end if

    indent_len = 0
    indent_str = ''
    if ( present(indent) ) then
      if ( len(indent) /= 0 ) then
        indent_len = len(indent)
        indent_str(1:indent_len) = indent
      end if
    end if

    !-----------------------------------------------------------------
    !  "PHYSPOLAY" ΰ
    !  Print the settings for "PHYSPOLAY"
    !-----------------------------------------------------------------
    if ( phy_spo_lay % initialized ) then
      call Printf( unit = out_unit, &                 ! (in)
        & fmt = indent_str(1:indent_len) // & 
        &       '#<PHYSPOLAY:: @initialized=%y', & ! (in)
        & l = (/ phy_spo_lay % initialized /) )   ! (in)

      call Printf( unit = out_unit, &             ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @imax=%d @jmax=%d @kmax=%d', &  ! (in)
        & i = (/ phy_spo_lay % imax, phy_spo_lay % jmax, &
        &        phy_spo_lay % kmax /) )          ! (in)

      call Printf( unit = out_unit, &                             ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @current_time=%f [sec] @delta_time=%f [sec]', & ! (in)
        & d = (/ EvalSec( phy_spo_lay % current_time ), &
        &        EvalSec( phy_spo_lay % delta_time ) /) )      ! (in)

      call Printf( unit = out_unit, &         ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @DampingTime=%f', &         ! (in)
        & d = (/phy_spo_lay % DampingTime/) ) ! (in)

      call Printf( unit = out_unit, &         ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @UpperPress=%f, @LowerPress=%f', &                   ! (in)
        & d = (/phy_spo_lay % UpperPress, phy_spo_lay % LowerPress/) ) ! (in)

      call PutLine( array = phy_spo_lay % x_Lon, & ! (in)
        & unit = out_unit, &                           ! (in)
        & lbounds = lbound(phy_spo_lay % x_Lon), & ! (in)
        & ubounds = ubound(phy_spo_lay % x_Lon), & ! (in)
        & indent = indent_str(1:indent_len) // &
        &          ' @x_Lon=' )                        ! (in)

      call PutLine( array = phy_spo_lay % y_Lat, & ! (in)
        & unit = out_unit, &                           ! (in)
        & lbounds = lbound(phy_spo_lay % y_Lat), & ! (in)
        & ubounds = ubound(phy_spo_lay % y_Lat), & ! (in)
        & indent = indent_str(1:indent_len) // &
        &          ' @y_Lat=' )                        ! (in)

      call PutLine( array = phy_spo_lay % z_Sigma, & ! (in)
        & unit = out_unit, &                         ! (in)
        & lbounds = lbound(phy_spo_lay % z_Sigma), & ! (in)
        & ubounds = ubound(phy_spo_lay % z_Sigma), & ! (in)
        & indent = indent_str(1:indent_len) // &
        &          ' @z_Sigma=' )                    ! (in)

      call IntAvrOprPutLine( phy_spo_lay % intavr_opr, & ! (in)
        & unit = out_unit, &                             ! (in)
        & indent = indent_str(1:indent_len) // ' ', &    ! (in)
        & err = err )                                    ! (out)

      call Printf( unit = out_unit, &         ! (in)
        & fmt = indent_str(1:indent_len) // &
        &       ' @gthstnml=' )               ! (in)
      call HstNmlInfoPutLine( &
        & gthstnml = phy_spo_lay % gthstnml, &    ! (in)
        & unit = out_unit, &                          ! (in)
        & indent = indent_str(1:indent_len) // '  ' ) ! (in)

      call Printf( unit = out_unit, &             ! (in)
        & fmt = indent_str(1:indent_len) // '>' ) ! (in)
    else
      call Printf( unit = out_unit, &                   ! (in)
        & fmt = indent_str(1:indent_len) // &           ! (in)
        &       '#<PHYSPOLAY:: @initialized=%y>', &  ! (in)
        & l = (/phy_spo_lay % initialized/) )       ! (in)
    end if

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  end subroutine PhySpoLayPutLine

  logical function PhySpoLayInitialized( phy_spo_lay ) result(result)
    !
    ! *phy_spo_lay* ꤵƤˤ .true. , 
    ! ꤵƤʤˤ .false. ֤ޤ. 
    !
    ! If *phy_spo_lay* is initialized, .true. is returned. 
    ! If *phy_spo_lay* is not initialized, .false. is returned. 
    !
    implicit none
    type(PHYSPOLAY), intent(in):: phy_spo_lay
  continue
    result = phy_spo_lay % initialized
  end function PhySpoLayInitialized

  subroutine PhySpoLaySetTime( phy_spo_lay, &
    & current_time_value, current_time_unit, &
    & err )
    !
    ! *phy_spo_lay* ФƻԤޤ. 
    !
    ! ҥȥǡϤƤˤ, 
    ! ҥȥǡνϻꤷޤ. 
    ! ٤Ǥ⤳Υ֥롼Ƥˤ, 
    ! ʸΥҥȥǡˤΥ֥롼ƤӽФ, 
    ! ԤäƤ. 
    ! ޤ, ǡϤ륵֥롼ФƤ 
    ! ץʥ historyput_flag  .true. ͿƤ. 
    !
    ! ʤ, Ϳ줿 *phy_spo_lay*  PhySpoLayCreate 
    ! ˤäƽꤵƤʤ, ץϥ顼ȯޤ. 
    !
    ! Set time to *phy_spo_lay*. 
    !
    ! When history data are output, 
    ! the output time of history data are specified. 
    ! Once this subroutine is called, the time of history data must be 
    ! specified by this routine before history data output. 
    ! In additional, give ".true." to an optional argument 
    ! "historyput_flag" of a data output subroutine. 
    !
    ! If *phy_spo_lay* is not initialized 
    ! by "PhySpoLayCreate" yet, error is occurred. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_date, only: DCDiffTimeCreate, EvalbyUnit
    use dc_error, only: StoreError, DC_NOERR, DC_ENOTINIT
    use gt4_history_nmlinfo, only: HstNmlInfoAdd, HstNmlInfoInquire, &
      & HstNmlInfoNames, HstNmlInfoAssocGtHist, &
      & HstNmlInfoOutputStepDisable, HstNmlInfoPutLine
    use gt4_history, only: GT_HISTORY, HistorySetTime, HistoryInitialized
    implicit none
    type(PHYSPOLAY), intent(inout):: phy_spo_lay
    real, intent(in):: current_time_value
                              ! ߻ο. Numerical value of current time
    character(*), intent(in):: current_time_unit
                              ! ߻ñ. Unit of current time
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    !-----------------------------------
    !  ҥȥեؤΥǡ
    !  Configure the settings for history data output
    character(STRING):: name = ''
                              ! ѿ̾. Variable identifier
    character(TOKEN):: interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(STRING):: varnames
                              ! ѿ̾ꥹ. 
                              ! List of variables
    character(TOKEN), pointer:: varnames_array(:) =>null()
                              ! ѿ̾ꥹ. 
                              ! List of variables (array) 
    integer:: i, vnmax
    type(GT_HISTORY), pointer:: gthist =>null()
                              ! gt4_history ⥸塼ѹ¤. 
                              ! Derived type for "gt4_history" module

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    character(*), parameter:: subname = 'PhySpoLaySetTime'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------
    if ( .not. phy_spo_lay % initialized ) then
      stat = DC_ENOTINIT
      cause_c = 'PHYSPOLAY'
      goto 999
    end if

    !-----------------------------------------------------------------
    !  
    !  Configure time
    !-----------------------------------------------------------------
    call DCDiffTimeCreate( &
      & diff = phy_spo_lay % current_time, &  ! (out)
      & value = real( current_time_value, DP ), & ! (in)
      & unit = current_time_unit )                ! (in)

    !-----------------------------------------------------------------
    !  ҥȥեؤΥǡλ
    !  Configure the time of history data
    !-----------------------------------------------------------------
    varnames = HstNmlInfoNames( phy_spo_lay % gthstnml )
    call Split( str = varnames, sep = ',', & ! (in)
      & carray = varnames_array )            ! (out)
    vnmax = size( varnames_array )

    do i = 1, vnmax
      name = varnames_array(i)
      if ( trim( name ) == '' ) exit

      call HstNmlInfoOutputStepDisable( & 
        & gthstnml = phy_spo_lay % gthstnml, & ! (inout)
        & name = name, &                           ! (in)
        & err = err )                              ! (out)

      nullify( gthist )
      call HstNmlInfoAssocGtHist( & 
        & gthstnml = phy_spo_lay % gthstnml, & ! (in)
        & name = name, &                           ! (in)
        & history = gthist, &                      ! (out)
        & err = err )                              ! (out)

      if ( HistoryInitialized( gthist ) ) then
        call HstNmlInfoInquire( &
          & gthstnml = phy_spo_lay % gthstnml, & ! (in)
          & name = name, &                           ! (in)
          & interval_unit = interval_unit )          ! (out)

        call HistorySetTime( &
          & history = gthist, &                     ! (inout)
          & time = &
          &   real( EvalbyUnit( phy_spo_lay % current_time, &
          &                     interval_unit) ) ) ! (in)
      end if

    end do

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  end subroutine PhySpoLaySetTime

  subroutine PhySpoLayNmlRead( nmlfile, &
    & DampingTime, UpperPress, LowerPress, &
    & gthstnml, &
    & err )
    !
    ! NAMELIST ե *nmlfile* ͤϤ뤿
    ! ֥롼Ǥ. PhySpoLayCreate 
    ! ǸƤӽФ뤳ȤꤷƤޤ. 
    !
    ! ͤ NAMELIST եǻꤵƤʤˤ, 
    ! Ϥ줿ͤΤޤ֤ޤ. 
    !
    ! ʤ, *nmlfile* ˶ʸͿ줿, ޤ
    ! Ϳ줿 *nmlfile* ɤ߹ळȤǤʤ, 
    ! ץϥ顼ȯޤ. 
    !
    ! This is a subroutine to input values from 
    ! NAMELIST file *nmlfile*. This subroutine is expected to be 
    ! called by "PhySpoLayCreate". 
    !
    ! A value not specified in NAMELIST file is returned 
    ! without change. 
    !
    ! If *nmlfile* is empty, or *nmlfile* can not be read, 
    ! error is occurred. 
    !
    use dc_trace, only: BeginSub, EndSub
    use dc_string, only: PutLine, Printf, Split, StrInclude, StoA, JoinChar
    use dc_types, only: DP, STRING, TOKEN, STDOUT
    use dc_iounit, only: FileOpen
    use dc_message, only: MessageNotify
    use dc_present, only: present_and_true
    use dc_date, only: DCDiffTimeCreate
    use dc_error, only: StoreError, DC_NOERR, DC_ENOFILEREAD, DC_ENOTINIT
    use gt4_history_nmlinfo, only: GTHST_NMLINFO, HstNmlInfoAdd, &
      & HstNmlInfoInquire, HstNmlInfoInitialized, HstNmlInfoPutLine
    implicit none
    character(*), intent(in):: nmlfile
                              ! NAMELIST ե̾. 
                              ! NAMELIST file name

    real(DP), intent(inout):: DampingTime
                              ! λ (ñ: ). 
                              ! Damping time constant (units: seconds)
    real(DP), intent(inout):: UpperPress
                              ! ϰϤξüε. 
                              ! Pressure in the top within the range of damping
    real(DP), intent(inout):: LowerPress
                              ! ϰϤβüε. 
                              ! Pressure at bottom within the range of damping
    type(GTHST_NMLINFO), intent(inout):: gthstnml
                              ! NAMELIST#phy_sponge_layer_history_nml 
                              ! ꤵ̤ΥǡϾ. 
                              ! 
                              ! ǥեͤʤɤ
                              ! ԤäͿ뤳. 
                              ! 
                              ! Individual data output information from 
                              ! "NAMELIST#phy_sponge_layer_history_nml". 
                              ! 
                              ! Before this argument is given to 
                              ! this procedure, initialize and 
                              ! configure the defaut settings. 
    logical, intent(out), optional:: err
                              ! 㳰ѥե饰. 
                              ! ǥեȤǤ, μ³ǥ顼
                              ! , ץ϶λޤ. 
                              !  *err* Ϳ, 
                              ! ץ϶λ, 
                              ! *err*  .true. ޤ. 
                              !
                              ! Exception handling flag. 
                              ! By default, when error occur in 
                              ! this procedure, the program aborts. 
                              ! If this *err* argument is given, 
                              ! .true. is substituted to *err* and 
                              ! the program does not abort. 

    namelist /phy_sponge_layer_nml/ &
      & DampingTime, UpperPress, LowerPress
                              ! phy_sponge_layer ⥸塼
                              ! NAMELIST ѿ̾. 
                              !
                              ! phy_sponge_layer#PhySpoLayCreate 
                              ! Ѥݤ, ץʥ *nmlfile* 
                              !  NAMELIST ե̾ꤹ뤳Ȥ, 
                              ! Υե뤫餳 NAMELIST ѿ
                              ! ɤ߹ߤޤ. 
                              !
                              ! NAMELIST group name for 
                              ! "phy_sponge_layer" module. 
                              ! 
                              ! If a NAMELIST filename is specified to 
                              ! an optional argument *nmlfile* when 
                              ! "phy_sponge_layer#PhySpoLayCreate" 
                              ! is used, this NAMELIST group is 
                              ! loaded from the file. 

    character(STRING):: name
                              ! ѿ̾. 
                              ! ξˤ, ¾ͤ
                              ! phy_sponge_layer ⥸塼ˤ
                              ! ϤǡƤ
                              ! ǥեͤȤʤޤ. 
                              ! 
                              ! "Data1,Data2" Τ褦˥ޤǶڤäʣ
                              ! ѿꤹ뤳ȤǽǤ. 
                              ! 
                              ! Variable identifier. 
                              ! If blank is given, other values are 
                              ! used as default values of output data 
                              ! in "phy_sponge_layer". 
                              ! 
                              ! Multiple variables can be specified 
                              ! as "Data1,Data2" too. Delimiter is comma. 
    character(STRING):: file
                              ! ϥե̾. 
                              ! ϥǥեͤȤƤϻѤޤ. 
                              ! *name* ͤꤵƤΤͭǤ. 
                              ! 
                              ! Output file name. 
                              ! This is not used as default value. 
                              ! This value is valid only when *name* is 
                              ! specified. 

    real:: interval_value
                              ! ҥȥǡνϴֳ֤ο. 
                              ! ͤͿ, Ϥ޻ߤޤ. 
                              ! Numerical value for interval of history data output
                              ! Negative values suppresses output.
    character(TOKEN):: interval_unit
                              ! ҥȥǡνϴֳ֤ñ. 
                              ! Unit for interval of history data output
    character(TOKEN):: precision
                              ! ҥȥǡ. 
                              ! Precision of history data
    logical:: average
                              ! ϥǡʿѲե饰. 
                              ! Flag for average of output data
    character(STRING):: fileprefix
                              ! ҥȥǡΥե̾Ƭ. 
                              ! Prefixes of history data filenames

    namelist /phy_sponge_layer_history_nml/ &
      & name, &
      & file, &
      & interval_value, &
      & interval_unit, &
      & precision, &
      & fileprefix, &
      & average
                              ! phy_sponge_layer ⥸塼Υҥȥǡ
                              ! NAMELIST ѿ̾. 
                              !
                              ! phy_sponge_layer#PhySpoLayCreate 
                              ! Ѥݤ, ץʥ *nmlfile* 
                              !  NAMELIST ե̾ꤹ뤳Ȥ, 
                              ! Υե뤫餳 NAMELIST ѿ
                              ! ɤ߹ߤޤ. 
                              !
                              ! NAMELIST group name for 
                              ! history data of "phy_sponge_layer" module. 
                              ! 
                              ! If a NAMELIST filename is specified to 
                              ! an optional argument *nmlfile* when 
                              ! "phy_sponge_layer#PhySpoLayCreate" 
                              ! is used, this NAMELIST group is 
                              ! loaded from the file. 

    !-----------------------------------
    !  ѿ
    !  Work variables
    integer:: stat
    character(STRING):: cause_c
    integer:: unit_nml        ! NAMELIST ե륪ץֹ. 
                              ! Unit number for NAMELIST file open
    integer:: iostat_nml      ! NAMELIST ɤ߹߻ IOSTAT. 
                              ! IOSTAT of NAMELIST read
    character(TOKEN):: pos_nml
                              ! NAMELIST ɤ߹߻Υե. 
                              ! File position of NAMELIST read
    character(*), parameter:: subname = 'PhySpoLayNmlRead'
  continue
    call BeginSub( subname )
    stat = DC_NOERR
    cause_c = ''

    !-----------------------------------------------------------------
    !  Υå
    !  Check initialization
    !-----------------------------------------------------------------

    !-----------------------------------------------------------------
    !  ʸ NAMELIST ѿ
    !  Substitute character arguments to NAMELIST group
    !-----------------------------------------------------------------
!!$    key00       = key00_

    !----------------------------------------------------------------
    !  NAMELIST եΥץ
    !  Open NAMELIST file
    !----------------------------------------------------------------
    call FileOpen( unit = unit_nml, & ! (out)
      & file = nmlfile, mode = 'r', & ! (in)
      & err = err )                   ! (out)
    if ( present_and_true(err) ) then
      stat = DC_ENOFILEREAD
      cause_c = nmlfile
      goto 999
    end if

    !-----------------------------------------------------------------
    !  NAMELIST ѿμ
    !  Get NAMELIST group
    !-----------------------------------------------------------------

    !-------------------------
    !  ʤɤμ
    !  Get coefficients etc.
    rewind( unit_nml )
    read( unit = unit_nml, &         ! (in)
      &    nml = phy_sponge_layer_nml, & ! (out)
      & iostat = iostat_nml )        ! (out)
    if ( iostat_nml == 0 ) then
      call MessageNotify( 'M', subname, &
        & 'NAMELIST group "%c" is loaded from "%c".', &
        & c1 = 'phy_sponge_layer_nml', c2 = trim(nmlfile) )
      write(STDOUT, nml = phy_sponge_layer_nml)
    else
      call MessageNotify( 'W', subname, &
        & 'NAMELIST group "%c" is not found in "%c" (iostat=%d).', &
        & c1 = 'phy_sponge_layer_nml', c2 = trim(nmlfile), &
        & i = (/iostat_nml/) )
    end if

    !-------------------------
    !  ϥǡθ̾μ
    !  Get individual information of output data
    rewind( unit_nml )
    iostat_nml = 0
    pos_nml = ''
    do while ( trim(pos_nml) /= 'APPEND' .and. iostat_nml == 0 )

      name                   = ''
      file                   = ''
      call HstNmlInfoInquire( &
        & gthstnml = gthstnml, &             ! (in)
        & interval_value = interval_value, & ! (out)
        & interval_unit = interval_unit, &   ! (out)
        & precision = precision, &           ! (out)
        & average = average, &               ! (out)
        & fileprefix = fileprefix )          ! (out)

      read( unit = unit_nml, &                 ! (in)
        &    nml = phy_sponge_layer_history_nml, & ! (out)
        & iostat = iostat_nml )                ! (out)
      inquire( unit = unit_nml, & ! (in)
        &  position = pos_nml )   ! (out)

      if ( iostat_nml == 0 ) then
        call MessageNotify( 'M', subname, &
          & 'NAMELIST group "%c" is loaded from "%c".', &
          & c1='phy_sponge_layer_history_nml', c2=trim(nmlfile) )
        write(STDOUT, nml = phy_sponge_layer_history_nml)

        call HstNmlInfoAdd( &
          & gthstnml = gthstnml, &             ! (in)
          & name = name, &                     ! (in)
          & file = file, &                     ! (in)
          & interval_value = interval_value, & ! (in)
          & interval_unit = interval_unit, &   ! (in)
          & precision = precision, &           ! (in)
          & average = average, &               ! (in)
          & fileprefix = fileprefix )          ! (in)

      else
        call MessageNotify( 'W', subname, &
          & 'NAMELIST group "%c" is not found in "%c" any more (iostat=%d).', &
          & c1='phy_sponge_layer_history_nml', c2=trim(nmlfile), &
          & i = (/iostat_nml/) )
      end if
    end do

    close( unit_nml )

    !-----------------------------------------------------------------
    !  NAMELIST ѿʸ
    !  Substitute NAMELIST group to character arguments
    !-----------------------------------------------------------------
!!$    key00_       = key00

    !-----------------------------------------------------------------
    !  λ, 㳰
    !  Termination and Exception handling
    !-----------------------------------------------------------------
999 continue
    call StoreError( stat, subname, err, cause_c )
    call EndSub( subname )

  end subroutine PhySpoLayNmlRead

end module phy_sponge_layer
